/*================================================================
  File:     CardinalityEstimates.sql

  Summary:  Taking a look at cardinality estimates gone wrong

  SQL Server Versions tested: SQL Server 2012, 2014
------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com

  (c) 2014, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/

USE [AdventureWorks2014];
GO

SELECT [d].[ProductID], COUNT([h].[SalesOrderID])
FROM [Sales].[SalesOrderDetailEnlarged] [d]
JOIN [Sales].[SalesOrderHeaderEnlarged] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [d].[ProductID] = 898
GROUP BY [d].[ProductID];

DBCC SHOW_STATISTICS ('Sales.SalesOrderDetailEnlarged', IX_SalesOrderDetailEnlarged_ProductID)
UPDATE STATISTICS Sales.SalesOrderDetailEnlarged IX_SalesOrderDetailEnlarged_ProductID WITH FULLSCAN;

SELECT [d].[ProductID], COUNT([h].[SalesOrderID])
FROM [Sales].[SalesOrderDetailEnlarged] [d]
JOIN [Sales].[SalesOrderHeaderEnlarged] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [d].[ProductID] = 898
GROUP BY [d].[ProductID]
OPTION (RECOMPILE);

UPDATE [Sales].[SalesOrderDetailEnlarged]
SET [OrderQty] = 1 
WHERE [SalesOrderID] = 51119

/*
	 Why is estimated rows <> actual rows for the key lookup?
*/
SELECT [d].[ProductID], COUNT([h].[SalesOrderID])
FROM [Sales].[SalesOrderDetailEnlarged] [d]
JOIN [Sales].[SalesOrderHeaderEnlarged] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [ProductID] = 727
GROUP BY [d].[ProductID]




/*
	create some tables for testing
*/
USE [master]
GO
ALTER DATABASE [AdventureWorks2014] SET AUTO_UPDATE_STATISTICS OFF WITH NO_WAIT;
GO

USE [AdventureWorks2014];
GO

SELECT TOP 0 *
INTO [Sales].[Copy_SalesOrderHeader]
FROM [Sales].[SalesOrderHeaderEnlarged];

ALTER TABLE [Sales].[Copy_SalesOrderHeader] 
ADD  CONSTRAINT [PK_Copy_SalesOrderHeader_SalesOrderID] PRIMARY KEY CLUSTERED 
(
	[SalesOrderID] ASC
)
GO

SELECT TOP 0 *
INTO [Sales].[Copy_SalesOrderDetail]
FROM [Sales].[SalesOrderDetailEnlarged];


ALTER TABLE [Sales].[Copy_SalesOrderDetail] 
ADD  CONSTRAINT [PK_Copy_SalesOrderDetail_SalesOrderID_SalesOrderDetailID] 
PRIMARY KEY CLUSTERED 
(
	[SalesOrderID] DESC,
	[SalesOrderDetailID] DESC
)
GO

CREATE NONCLUSTERED INDEX [IX_Copy_SalesOrderDetail_ProductID] ON [Sales].[Copy_SalesOrderDetail]
(
	[ProductID] ASC
)
GO

/*
	load some data
*/
INSERT INTO [Sales].[Copy_SalesOrderHeader]
    ([RevisionNumber],[OrderDate],[DueDate],[ShipDate],[Status],[OnlineOrderFlag],[SalesOrderNumber]
	,[PurchaseOrderNumber],[AccountNumber],[CustomerID],[SalesPersonID],[TerritoryID],[BillToAddressID]
	,[ShipToAddressID],[ShipMethodID],[CreditCardID],[CreditCardApprovalCode],[CurrencyRateID],[SubTotal],[TaxAmt]
	,[Freight],[TotalDue],[Comment],[rowguid],[ModifiedDate])
SELECT 	[RevisionNumber],[OrderDate],[DueDate],[ShipDate],[Status],[OnlineOrderFlag],[SalesOrderNumber]
	,[PurchaseOrderNumber],[AccountNumber],[CustomerID],[SalesPersonID],[TerritoryID],[BillToAddressID]
	,[ShipToAddressID],[ShipMethodID],[CreditCardID],[CreditCardApprovalCode],[CurrencyRateID],[SubTotal],[TaxAmt]
	,[Freight],[TotalDue],[Comment],[rowguid],[ModifiedDate]
FROM [Sales].[SalesOrderHeaderEnlarged]
WHERE SalesOrderID <= 143657;

INSERT INTO [Sales].[Copy_SalesOrderDetail]
([SalesOrderID],[CarrierTrackingNumber],[OrderQty],[ProductID],[SpecialOfferID],[UnitPrice]
,[UnitPriceDiscount],[LineTotal],[rowguid],[ModifiedDate])
SELECT [a].[SalesOrderID],[a].[CarrierTrackingNumber],[a].[OrderQty],[b].[ProductID],[a].[SpecialOfferID]
,[a].[UnitPrice],[a].[UnitPriceDiscount],[a].[LineTotal],[a].[rowguid],[a].[ModifiedDate]
FROM [Sales].[SalesOrderDetailEnlarged] [a]
JOIN [Sales].[SalesOrderDetail] [b] ON  [a].[SalesOrderID] = [b].[SalesOrderID]
WHERE [a].[SalesOrderID] <= 143657
AND [b].[ProductID] BETWEEN 750 AND 850
ORDER BY [ProductID];

/*
	update stats
*/
UPDATE STATISTICS [Sales].[Copy_SalesOrderHeader] WITH SAMPLE 10 PERCENT;
UPDATE STATISTICS [Sales].[Copy_SalesOrderDetail] WITH SAMPLE 10 PERCENT;

DBCC SHOW_STATISTICS ('Sales.Copy_SalesOrderHeader',PK_Copy_SalesOrderHeader_SalesOrderID) WITH STAT_HEADER;
GO
DBCC SHOW_STATISTICS ('Sales.Copy_SalesOrderDetail',PK_Copy_SalesOrderDetail_SalesOrderID_SalesOrderDetailID) WITH STAT_HEADER;
GO
DBCC SHOW_STATISTICS ('Sales.Copy_SalesOrderDetail',IX_Copy_SalesOrderDetail_ProductID)WITH STAT_HEADER;
GO

SELECT [d].[ProductID], COUNT([h].[SalesOrderID])
FROM [Sales].[Copy_SalesOrderDetail] [d]
JOIN [Sales].[Copy_SalesOrderHeader] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [d].[ProductID] = 757
GROUP BY [d].[ProductID];

/*
	finish populating the tables
*/
INSERT INTO [Sales].[Copy_SalesOrderHeader]
    ([RevisionNumber],[OrderDate],[DueDate],[ShipDate],[Status],[OnlineOrderFlag],[SalesOrderNumber]
	,[PurchaseOrderNumber],[AccountNumber],[CustomerID],[SalesPersonID],[TerritoryID],[BillToAddressID]
	,[ShipToAddressID],[ShipMethodID],[CreditCardID],[CreditCardApprovalCode],[CurrencyRateID],[SubTotal],[TaxAmt]
	,[Freight],[TotalDue],[Comment],[rowguid],[ModifiedDate])
SELECT 	[RevisionNumber],[OrderDate],[DueDate],[ShipDate],[Status],[OnlineOrderFlag],[SalesOrderNumber]
	,[PurchaseOrderNumber],[AccountNumber],[CustomerID],[SalesPersonID],[TerritoryID],[BillToAddressID]
	,[ShipToAddressID],[ShipMethodID],[CreditCardID],[CreditCardApprovalCode],[CurrencyRateID],[SubTotal],[TaxAmt]
	,[Freight],[TotalDue],[Comment],[rowguid],[ModifiedDate]
FROM [Sales].[SalesOrderHeaderEnlarged]
WHERE SalesOrderID > 143657;

INSERT INTO [Sales].[Copy_SalesOrderDetail]
([SalesOrderID],[CarrierTrackingNumber],[OrderQty],[ProductID],[SpecialOfferID],[UnitPrice]
,[UnitPriceDiscount],[LineTotal],[rowguid],[ModifiedDate])
SELECT [a].[SalesOrderID],[a].[CarrierTrackingNumber],[a].[OrderQty],[a].[ProductID],[a].[SpecialOfferID]
,[a].[UnitPrice],[a].[UnitPriceDiscount],[a].[LineTotal],[a].[rowguid],[a].[ModifiedDate]
FROM [Sales].[SalesOrderDetailEnlarged] [a]
WHERE [a].[SalesOrderID] > 143657

/*
	run our query with some different values
*/
SELECT [d].[ProductID], COUNT([h].[SalesOrderID])
FROM [Sales].[Copy_SalesOrderDetail] [d]
JOIN [Sales].[Copy_SalesOrderHeader] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [d].[ProductID] = 757
GROUP BY [d].[ProductID]

SELECT [d].[ProductID], COUNT([h].[SalesOrderID])
FROM [Sales].[Copy_SalesOrderDetail] [d]
JOIN [Sales].[Copy_SalesOrderHeader] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [d].[ProductID] = 782
GROUP BY [d].[ProductID]

SELECT [d].[ProductID], COUNT([h].[SalesOrderID])
FROM [Sales].[Copy_SalesOrderDetail] [d]
JOIN [Sales].[Copy_SalesOrderHeader] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [d].[ProductID] = 870
GROUP BY [d].[ProductID]

/*
	update stats, re-run those three queries
*/
UPDATE STATISTICS [Sales].[Copy_SalesOrderHeader] WITH FULLSCAN;
UPDATE STATISTICS [Sales].[Copy_SalesOrderDetail] WITH FULLSCAN;


/*
	clean up
*/
DROP TABLE [Sales].[Copy_SalesOrderHeader] 
DROP TABLE [Sales].[Copy_SalesOrderDetail] 

USE [master]
GO
ALTER DATABASE [AdventureWorks2014] SET AUTO_UPDATE_STATISTICS ON WITH NO_WAIT;
GO
