/*================================================================
  File:     02_NestedLoop.sql

  Summary:  Nested loop example with a large number of 
			random IOs on the inner loop

  SQL Server Versions tested: SQL Server 2012, 2014
------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com

  (c) 2014, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/
USE [AdventureWorks2014];
GO

/*
	General query that uses a nested loop
*/
SELECT [d].[ProductID], COUNT([h].[SalesOrderID])
FROM [Sales].[SalesOrderDetailEnlarged] [d]
JOIN [Sales].[SalesOrderHeaderEnlarged] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [d].[ProductID] = 898
GROUP BY [d].[ProductID];


DBCC SHOW_STATISTICS ('Sales.SalesOrderDetailEnlarged', IX_SalesOrderDetailEnlarged_ProductID)


/*
	Create SP
*/
IF OBJECT_ID ( 'Sales.usp_GetProductCount', 'P' ) IS NOT NULL 
    DROP PROCEDURE [Sales].[usp_GetProductCount];
GO

CREATE PROCEDURE [Sales].[usp_GetProductCount]
	@ProductID INT
AS	

	SELECT [d].[ProductID], COUNT([h].[SalesOrderID])
	FROM [Sales].[SalesOrderDetailEnlarged] [d]
	JOIN [Sales].[SalesOrderHeaderEnlarged] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
	WHERE [d].[ProductID] = @ProductID
	GROUP BY [d].[ProductID];

GO


/*
	Run SP a few times
*/
dbcc freeproccache
EXEC [Sales].[usp_GetProductCount] 898;

EXEC [Sales].[usp_GetProductCount] 956;

EXEC [Sales].[usp_GetProductCount] 870;


/*
	Need to look at more than just the plan...
*/
SET STATISTICS IO ON;
SET STATISTICS TIME ON;
EXEC [Sales].[usp_GetProductCount] 898;
EXEC [Sales].[usp_GetProductCount] 870;


/*
	What if I run the plans separately?
*/
EXEC [Sales].[usp_GetProductCount] 898 WITH RECOMPILE;
EXEC [Sales].[usp_GetProductCount] 870 WITH RECOMPILE;

EXEC sp_recompile '[Sales].[usp_GetProductCount]';

/*
	clean up
*/
DROP PROCEDURE [Sales].[usp_GetProductCount]


/*
	 Why is estimated rows <> actual rows for the key lookup?
*/
SELECT [d].[ProductID], COUNT([h].[SalesOrderID])
FROM [Sales].[SalesOrderDetailEnlarged] [d]
JOIN [Sales].[SalesOrderHeaderEnlarged] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [ProductID] = 727
GROUP BY [d].[ProductID]




SELECT [d].[ProductID], COUNT([h].[SalesOrderID])
FROM [Sales].[SalesOrderDetailEnlarged] [d]
JOIN [Sales].[SalesOrderHeaderEnlarged] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [d].[ProductID] = 870
GROUP BY [d].[ProductID]