/*================================================================
  File:     04_ResidualPredicates.sql

  Summary:  Examine queries with predicates, filters and residuals

  SQL Server Versions tested: SQL Server 2012, 2014
------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com

  (c) 2014, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/
USE [AdventureWorks2014];
GO

SET STATISTICS IO ON;

/*
	Check the predicate
*/
SELECT [ProductID], [Name], [ReorderPoint]
FROM [Production].[Product]
WHERE [ReorderPoint] > 500 and ReorderPoint < 700;

/*
	Let's add an index on reorderpoint
*/
CREATE NONCLUSTERED
INDEX [NCI_Product_ReorderPoint] ON [Production].[Product] (
	[ReorderPoint]
	)
INCLUDE ([Name]);
GO

/*
	Did we lose our residual?
*/
SELECT [ProductID], [Name], [ReorderPoint]
FROM [Production].[Product]
WHERE [ReorderPoint] > 500 and ReorderPoint < 700;


/*
	Do we still have a seek predidcate
	with an IN?
*/
SELECT [ProductID], [Name], [ReorderPoint]
FROM [Production].[Product]
WHERE [ReorderPoint] IN (375, 600, 750)


/*
	seek predicate
*/
SELECT 
	[e].[NationalIDNumber],
	[p].[Title],
	[p].[FirstName],
	[p].[MiddleName],
	[p].[LastName],
	[p].[Suffix],
	[e].[HireDate]
FROM [HumanResources].[Employee] [e]
JOIN [Person].[Person] [p] ON [p].[BusinessEntityID] = [e].[BusinessEntityID];

/*
	seek predicate and a predicate
*/
SELECT 
	[e].[NationalIDNumber],
	[p].[Title],
	[p].[FirstName],
	[p].[MiddleName],
	[p].[LastName],
	[p].[Suffix],
	[e].[HireDate]
FROM [HumanResources].[Employee] [e]
JOIN [Person].[Person] [p] ON [p].[BusinessEntityID] = [e].[BusinessEntityID]
WHERE [p].[LastName] = 'Williams';


/*
	we can also find residuals in lookups 
*/
SELECT soh.CustomerID, sod.ProductID
FROM Sales.SalesOrderDetail sod
INNER JOIN  Sales.SalesOrderHeader soh on sod.SalesOrderID = soh.SalesOrderID
WHERE soh.customerid = 29712;

SELECT soh.CustomerID, sod.ProductID
FROM Sales.SalesOrderDetail sod
INNER JOIN  Sales.SalesOrderHeader soh on sod.SalesOrderID = soh.SalesOrderID
WHERE soh.customerid = 29712
AND ShipMethodID = 5;


/*
	USE TF 9130 to push this out to a filter to see it better
	(don't use in production code)
*/
SELECT soh.CustomerID, sod.ProductID
FROM Sales.SalesOrderDetail sod
INNER JOIN  Sales.SalesOrderHeader soh on sod.SalesOrderID = soh.SalesOrderID
WHERE soh.customerid = 29712
AND ShipMethodID = 5
OPTION (QUERYTRACEON 9130)



/*
	and just in case, you can also find them in hash matches...
*/
SELECT [s].[Name], COUNT(*)
FROM [Production].[ProductSubcategory] s
INNER JOIN [Production].[Product] p
ON [s].[ProductSubcategoryID] = [p].[ProductSubcategoryID]
WHERE RIGHT([p].[Name], LEN([s].[Name])) like '%Fork'
GROUP BY [s].[Name]; 

SELECT [s].[Name], COUNT(*)
FROM [Production].[ProductSubcategory] s
INNER JOIN [Production].[Product] p
ON [s].[ProductSubcategoryID] = [p].[ProductSubcategoryID]
WHERE RIGHT([p].[Name], LEN([s].[Name])) like '%Fork'
GROUP BY [s].[Name]
OPTION (MERGE JOIN); 


/*
	clean up
*/
DROP INDEX [Production].[Product].[NCI_Product_ReorderPoint];