/*================================================================
  File:     05_ComputeScalar.sql

  Summary:  Find hidden costs in the compute scalar operator

  SQL Server Version 2012, 2014
------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com

  (c) 2014, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/


USE [AdventureWorks2014];
GO

SELECT COUNT(*)
FROM [Sales].[SalesOrderHeaderEnlarged] AS [h];

SELECT COUNT_BIG(*)
FROM [Sales].[SalesOrderHeaderEnlarged] AS [h];

/*
	Create a scalar UDF that counts how many different products each salesperson has sold. 
*/
CREATE FUNCTION dbo.CountProductsSold (
	@SalesPersonID INT
	) RETURNS INT

AS

BEGIN
    DECLARE @SoldCount INT;
	SELECT @SoldCount = COUNT(DISTINCT [d].[ProductID])
	FROM [Sales].[SalesOrderHeader] AS [h]
	JOIN [Sales].[SalesOrderDetail] AS [d] ON [d].[SalesOrderID] = [h].[SalesOrderID]
	WHERE [h].[SalesPersonID] = @SalesPersonID;
	
    RETURN (@SoldCount);

END

GO


/*
	What do we see in the plan?
*/
SELECT 
	[sp].[BusinessEntityID] AS [SalesPerson], 
	[dbo].[CountProductsSold]([sp].[BusinessEntityID]) AS [NumberOfProductsSold]
FROM [Sales].[SalesPerson] AS [sp];




/*
	re-run with STATISTICS IO enabled
*/	
SET STATISTICS IO ON;
SELECT 
	[sp].[BusinessEntityID] AS [SalesPerson], 
	[dbo].[CountProductsSold]([sp].[BusinessEntityID]) AS [NumberOfProductsSold]
FROM [Sales].[SalesPerson] AS [sp];
SET STATISTICS IO OFF;



/*
	How do you see what's going on?
	XE 
	**set session_id**
*/
 
CREATE EVENT SESSION [CaptureQueryIO] 
	ON SERVER 
ADD EVENT sqlserver.sp_statement_completed(SET collect_statement=(1)
    WHERE ([sqlserver].[session_id]=(57))) ,
ADD EVENT sqlserver.sql_statement_completed(
    WHERE ([sqlserver].[session_id]=(57)))
ADD TARGET package0.event_file(SET filename=N'C:\temp\CaptureQueryIO.xel')
WITH (MAX_MEMORY=4096 KB,EVENT_RETENTION_MODE=ALLOW_SINGLE_EVENT_LOSS,MAX_DISPATCH_LATENCY=5 SECONDS,
MAX_EVENT_SIZE=0 KB,MEMORY_PARTITION_MODE=NONE,TRACK_CAUSALITY=ON,STARTUP_STATE=OFF)
GO

ALTER EVENT SESSION [CaptureQueryIO]
ON SERVER
STATE = START;

SELECT 
	[BusinessEntityID] AS [SalesPerson], 
	[dbo].[CountProductsSold]([BusinessEntityID]) AS [NumberOfProductsSold]
FROM [Sales].[SalesPerson];

ALTER EVENT SESSION [CaptureQueryIO]
ON SERVER
STATE = STOP;
/*
	check XE data
*/


/*
	How to re-write?
*/
SELECT 
	[sp].[BusinessEntityID] AS [SalesPerson], 
	(SELECT COUNT(DISTINCT [d].[ProductID])
	FROM [Sales].[SalesOrderHeader] AS [h]
	JOIN [Sales].[SalesOrderDetail] AS [d] ON [d].[SalesOrderID] = [h].[SalesOrderID]
	WHERE [h].[SalesPersonID] = [sp].[BusinessEntityID] )AS [NumberOfProductsSold]
	--[dbo].[CountProductsSold]([sp].[BusinessEntityID]) 
FROM [Sales].[SalesPerson] AS [sp];


SELECT 
	[BusinessEntityID] AS [SalesPerson], 
	COUNT(DISTINCT [d].[ProductID]) AS [NumberOfProductsSold]
FROM [Sales].[SalesPerson] [p]
JOIN[Sales].[SalesOrderHeader] AS [h] ON [p].[BusinessEntityID] = [h].[SalesPersonID]
JOIN [Sales].[SalesOrderDetail] AS [d] ON [d].[SalesOrderID] = [h].[SalesOrderID]
GROUP BY [BusinessEntityID];
GO

SELECT 
	[BusinessEntityID] AS [SalesPerson], 
	[dbo].[CountProductsSold]([BusinessEntityID]) AS [NumberOfProductsSold]
FROM [Sales].[SalesPerson];
GO


/*
	measure perf difference
*/
ALTER EVENT SESSION [CaptureQueryIO]
ON SERVER
STATE = START;

SELECT 
	[BusinessEntityID] AS [SalesPerson], 
	COUNT(DISTINCT [d].[ProductID]) AS [NumberOfProductsSold]
FROM [Sales].[SalesPerson] [p]
JOIN[Sales].[SalesOrderHeader] AS [h] ON [p].[BusinessEntityID] = [h].[SalesPersonID]
JOIN [Sales].[SalesOrderDetail] AS [d] ON [d].[SalesOrderID] = [h].[SalesOrderID]
GROUP BY [BusinessEntityID];
GO

SELECT 
	[BusinessEntityID] AS [SalesPerson], 
	[dbo].[CountProductsSold]([BusinessEntityID]) AS [NumberOfProductsSold]
FROM [Sales].[SalesPerson];
GO

ALTER EVENT SESSION [CaptureQueryIO]
ON SERVER
STATE = STOP;

/*
	clean up
*/
DROP EVENT SESSION [CaptureQueryIO] ON SERVER;
DROP FUNCTION dbo.CountProductsSold;